/*"Gear Indicator Dll": Simple OutGauge server for Directx 9 games
 * Author: Vladimir Kadlec
 *         vladimirkadlec@gmail.com
 *         http://www.fi.muni.cz/~xkadlec
 */
/*
This file is part of Gear Indicator Dll.

Gear Indicator Dll is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Gear Indicator Dll is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Gear Indicator Dll; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <windows.h>
#include "log.h"
#include "config.h"

#define MAX_IP_SIZE 100

void init_config (Config *conf) {
int i;
    conf->ip = (char*)calloc (MAX_IP_SIZE, 1);
    strcpy (conf->ip, "127.0.0.1");
    conf->port = 11111;
    conf->out_skt = -1;
    conf->proc_handle = NULL;
    conf->car_struct = (RBRCarInfo**) 0x0165FC68;
    conf->car_slot = (unsigned int*) 0x7eac4c;
    conf->debug = 0;
    conf->start_time = clock();
    conf->hmodule = NULL;
    for (i = 0; i < NUMBER_OF_CARS; i++) {
        conf->shiftl[i] = 4000.0;
    }
}

void print_config (Config *conf) {
int i;

    log ("Current configuration:\n");
    log ("IP:%s Port:%d\n", conf->ip, conf->port);
    log ("Car Struct Address Ptr: 0x%x\n", conf->car_struct);
    log ("Rpm ptr: 0x%x\nGear ptr: 0x%x\n",
        &((*conf->car_struct)->rpm), &((*conf->car_struct)->gear_number));
    log ("Shift light levels:");
    for (i = 0; i < NUMBER_OF_CARS; i++) {
        log (" %.0f", conf->shiftl[i]);
    }
    log ("\n");
}

void read_line (char *file_buf, long *file_pos, char **line_buf)
{
    long i;
    *line_buf = file_buf + (*file_pos);
    i = *file_pos;                                                                  while ((file_buf[i] != '\n') && (file_buf[i] != 0))
        i++;
    file_buf[i] = 0;
    *file_pos = i + 1;
}

typedef enum {S_NONE, S_IP, S_PORT, S_SHIFTL, S_SHIFTL_N,
S_DEBUG, S_CARS_ADDR, S_GEAR_OFFSET} STATE;


int read_config (char *file_name, Config *conf) {
FILE *f;
char *file_buf, *line_buf, *wrd;
long file_size, file_pos = 0, line_no = 0;
STATE state = S_NONE;
int port;
char *err_port;
int in_pin = 0;
int shiftl_no = -1;
word lfs_flags;

    if ((f = fopen (file_name, "r")) == NULL) {
        error ("Can't open file: %s.\n", file_name);
        return (0);
    }
    fseek (f, 0, SEEK_END);
    file_size = ftell (f);
    fseek (f, 0, SEEK_SET);
    if ((file_buf = (char *) calloc (file_size+1, 1)) == NULL) {
        error ("Can't allocate memory for config file.\n");
        return (0);
    }
    file_size = fread (file_buf, 1, file_size, f);
    fclose (f);

    while (file_pos < file_size) {
        if (state != S_NONE) {
            if (state != S_SHIFTL_N) {
                error ("read_config, invalid state, please report.\n");
                free (file_buf);
                return (0);
            }
            warning ("read_config, line: %d,"
                           " too litle numbers for display\n", line_no);
        }
        read_line (file_buf, &file_pos, &line_buf);
        line_no++;
        state = S_NONE;
        wrd = strtok (line_buf, " \t\n\r");
        for (; 1; wrd = strtok (NULL, " \t\n\r")) {
            if (wrd == NULL)
                break;
            if (wrd[0] == 0)
                continue;
            if (wrd[0] == '#')
                break;
            switch (state) {
            case S_DEBUG:
               port = strtol (wrd, &err_port, 10);
                if (((*err_port) != 0) || ((port != 0) && (port != 1))) {
                    warning ("read_config, line: %d,"
                              " Invalid debug option (should be 1 or 0)\n",
                              line_no);
                    state = S_NONE;
                    continue;
                }
                conf->debug = port;
                state = S_NONE;
            break;
            case S_IP:
               strncpy (conf->ip, wrd, MAX_IP_SIZE);
               state = S_NONE;
            break;
            case S_PORT:
                port = strtol (wrd, &err_port, 10);
                if ((*err_port) != 0) {
                    warning ("read_config, line: %d, Invalid port number\n",
                             line_no);
                    state = S_NONE;
                    continue;
                }
                conf->port = port;
                state = S_NONE;
            break;
            case S_SHIFTL:
                shiftl_no = 0;
                port = strtol (wrd, &err_port, 10);
                if ((*err_port) != 0) {
                    warning ("read_config, line: %d, Invalid shift light level\n",
                             line_no);
                    state = S_NONE;
                    continue;
                }
                conf->shiftl[shiftl_no] = port;
                shiftl_no++;
                state = S_SHIFTL_N;
            break;
            case S_SHIFTL_N:
                port = strtol (wrd, &err_port, 10);
                if ((*err_port) != 0) {
                    warning ("read_config, line: %d, Invalid shift light level\n",
                             line_no);
                    state = S_NONE;
                    continue;
                }
                conf->shiftl[shiftl_no] = port;
                shiftl_no++;
                if (shiftl_no == NUMBER_OF_CARS) state = S_NONE;
            break;
            case S_NONE:
               if (strcasecmp (wrd, "IP") == 0) {
                   state = S_IP;
                   continue;
               }
               if (strcasecmp (wrd, "Port") == 0) {
                   state = S_PORT;
                   continue;
               }
               if (strcasecmp (wrd, "Debug") == 0) {
                   state = S_DEBUG;
                   continue;
               }
               if (strcasecmp (wrd, "Shift_Light_Levels") == 0) {
                   state = S_SHIFTL;
                   continue;
               }
               warning ("read_config, line: %d, Unknown token \"%s\"\n",
                             line_no, wrd);
            break;
            default:
                error ("read_config, undefined state!");
                free (file_buf);
                return (0);
            }
        }
    }
    free (file_buf);
    return (1);
}
