/*"Gear Indicator Dll": Simple OutGauge server for Directx 9 games
 * Author: Vladimir Kadlec
 *         vladimirkadlec@gmail.com
 *         http://www.fi.muni.cz/~xkadlec
/*
This file is part of Gear Indicator Dll. The code is based on the
"Proxy-Dll" concept by Michael Koch, see http://www.mikoweb.eu

Gear Indicator Dll is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Gear Indicator Dll is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Gear Indicator Dll; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
// proxydll.cpp
#include <windows.h>
#include <string.h>

#include "stdafx.h"
#include "proxydll.h"

// global variables
#pragma data_seg (".d3d9_shared")
myIDirect3DSwapChain9*  gl_pmyIDirect3DSwapChain9 = NULL;
myIDirect3DDevice9*		gl_pmyIDirect3DDevice9 = NULL;
myIDirect3D9*			gl_pmyIDirect3D9 = NULL;
HINSTANCE				gl_hOriginalDll = NULL;
HINSTANCE				gl_hThisInstance = NULL;

//config + sockets.... for outgauge
int OUT_SKT;
Config CONFIG;
OutGaugePacket OG_PACKET;
#pragma data_seg ()

void error_exit() {
	error("exit called\n");
	::ExitProcess(0);
}
     
int init_gidll (int *in_skt, Config *conf) {
struct hostent *localhost;
struct sockaddr_in server_addr;

WORD wVersionRequested = MAKEWORD(1,1); // slo verze
WSADATA wsadata;                        // Struktura s info. o knihovn;

    init_config (conf);
    if (!read_config ("gidll_config.txt", conf)) {
        error ("Read_config\n");
        error_exit();
    }
    print_config (conf);

    // Pipravme sokety na prci
    if (WSAStartup(wVersionRequested, &wsadata) != 0) {
        error ("Init socket.\n");
        error_exit();
    }
	localhost = gethostbyname (conf->ip);
	if (localhost == NULL) {
		error ("Can't resolve IP address: %s\n", conf->ip);
        WSACleanup();
		error_exit ();
	}

	*in_skt = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (*in_skt == INVALID_SOCKET) {
		error ("Create socket.\n");
        WSACleanup();
		error_exit ();
	}
	
	server_addr.sin_family = AF_INET;
    server_addr.sin_port = htons (conf->port); //hack
    memcpy(&(server_addr.sin_addr), localhost->h_addr, localhost->h_length);

	//zadny pripojeni v tehle chvili, pracujem s UDP, je to jen prirazeni
	//adresy

    log ("Going to connect...\n");
	if (connect (*in_skt, (struct sockaddr*)&server_addr,
				sizeof(server_addr)) == -1) {
		error ("connect()\n");
        WSACleanup();
		error_exit ();
    }
    log ("Connect success.\n");
    conf->out_skt = *in_skt;
    conf->proc_handle = GetCurrentProcess();
    conf->hmodule = GetModuleHandle (NULL);
    memset (&OG_PACKET, 0, sizeof (OutGaugePacket));
    OG_PACKET.Fuel = 1.0;
    OG_PACKET.Car[0] = 'R'; //we'll fill Car with RR<slot>
    OG_PACKET.Car[1] = 'R';
    log ("Init return\n");
    return (1);
}

int done_gidll (int *in_skt) {
    log ("Done GI Dll called\n");
    closesocket (*in_skt);
    WSACleanup();
    return (1);
}

BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	// to avoid compiler lvl4 warnings 
    LPVOID lpDummy = lpReserved;
    lpDummy = NULL;
    switch (ul_reason_for_call)
	{
	    case DLL_PROCESS_ATTACH: InitInstance(hModule); break;
	    case DLL_PROCESS_DETACH: ExitInstance(); break;
        
        case DLL_THREAD_ATTACH:  break;
	    case DLL_THREAD_DETACH:  break;
	}
    return TRUE;
}

// Exported function (faking d3d9.dll's one-and-only export)
__declspec(dllexport) IDirect3D9* WINAPI Direct3DCreate9(UINT SDKVersion)
{
	if (!gl_hOriginalDll) LoadOriginalDll(); // looking for the "right d3d9.dll"
	
	// Hooking IDirect3D Object from Original Library
	typedef IDirect3D9 *(WINAPI* D3D9_Type)(UINT SDKVersion);
	D3D9_Type D3DCreate9_fn = (D3D9_Type) GetProcAddress( gl_hOriginalDll, "Direct3DCreate9");
    
    // Debug
	if (!D3DCreate9_fn) 
    {
        OutputDebugString("PROXYDLL: Pointer to original D3DCreate9 function not received ERROR ****\r\n");
        ::ExitProcess(0); // exit the hard way
    }
	
	//GI init
	set_logfile ("gidll_log.txt");
	log ("Direct3DCreate9 called.\n");
	init_gidll (&OUT_SKT, &CONFIG);

	// Request pointer from Original Dll. 
	IDirect3D9 *pIDirect3D9_orig = D3DCreate9_fn(SDKVersion);
	
	// Create my IDirect3D8 object and store pointer to original object there.
	// note: the object will delete itself once Ref count is zero (similar to COM objects)
	gl_pmyIDirect3D9 = new myIDirect3D9 (pIDirect3D9_orig);
    	
	// Return pointer to hooking Object instead of "real one"
	log ("Direct3DCreate9 return\n");
	return (gl_pmyIDirect3D9);
}

void InitInstance(HANDLE hModule) 
{
	OutputDebugString("PROXYDLL: InitInstance called.\r\n");
	
	// Initialisation
	gl_hOriginalDll				= NULL;
	gl_hThisInstance			= NULL;
	gl_pmyIDirect3D9			= NULL;
	gl_pmyIDirect3DDevice9		= NULL;
	gl_pmyIDirect3DSwapChain9	= NULL;
	
	// Storing Instance handle into global var
	gl_hThisInstance = (HINSTANCE)  hModule;
	
	warning ("Init instence called\n");
}

void LoadOriginalDll(void)
{
    char buffer[MAX_PATH];
    
    // Getting path to system dir and to d3d8.dll
	::GetSystemDirectory(buffer,MAX_PATH);

	// Append dll name
	strcat(buffer,"\\d3d9.dll");
	
	// try to load the system's d3d9.dll, if pointer empty
	if (!gl_hOriginalDll) gl_hOriginalDll = ::LoadLibrary(buffer);

	// Debug
	if (!gl_hOriginalDll)
	{
		OutputDebugString("PROXYDLL: Original d3d9.dll not loaded ERROR ****\r\n");
		::ExitProcess(0); // exit the hard way
	}
}

void ExitInstance() 
{    
    OutputDebugString("PROXYDLL: ExitInstance called.\r\n");
	
	// Release the system's d3d9.dll
	if (gl_hOriginalDll)
	{
		::FreeLibrary(gl_hOriginalDll);
	    gl_hOriginalDll = NULL;  
	}
	warning ("Exit instence called\n");
	done_gidll (&OUT_SKT);
}

